/****************************************************************************
*
*   Copyright (c) 2011 www.roborealm.com
*   (please link back if you use this code!)
*
*   This program is free software; you can redistribute it and/or modify
*   it for whatever needs you may have.
*
****************************************************************************/

#include "SoR_Utils.h" //includes all the technical stuff

extern u08 uartReadyTx[4];

#define MCU_GET_ID 1
#define MCU_END_FRAME 2
#define MCU_SET_NUMBER_MAILBOX 3
#define MCU_SET_TEXT_MAILBOX 4
#define MCU_SET_LOGIC_MAILBOX 5
#define MCU_GET_NUMBER_MAILBOX 6
#define MCU_GET_TEXT_MAILBOX 7
#define MCU_GET_LOGIC_MAILBOX 8

#define UART0 0
#define UART1 1
#define UART2 2
#define UART3 3

unsigned char mailboxToSend[20];
unsigned char mailboxSendFlag[20];

unsigned long intValue[20];
unsigned char logicValue[20];
unsigned char *textValue[20];

unsigned char textStore[1024];
int textStoreTop=0;

// serial send buffer.
unsigned char send_buffer[32];

void processMailboxes();

// serial receive buffer: A ring buffer that we will use to receive bytes on USB_COMM.
char receive_buffer[32];
// receive_buffer_position: This variable will keep track of which bytes in the receive buffer
// we have already processed.
unsigned char receive_buffer_bottom = 0;

unsigned char receive_buffer_top = 0;

// is called when a new character from the PC is available
void receive_data(unsigned char c)
{
	receive_buffer[receive_buffer_top++]=c;
	if (receive_buffer_top>=32) receive_buffer_top=0;

	sei();//turn on interrupts
}

unsigned char getByte()
{
	while (receive_buffer_bottom==receive_buffer_top)
		delay_cycles(100);

	unsigned char res = receive_buffer[receive_buffer_bottom++];

	// Increment receive_buffer_position, but wrap around when it gets to
	// the end of the buffer.
	if (receive_buffer_bottom>=32) receive_buffer_bottom = 0;

	return res;
}

void write(char *buffer, int len)
{
	int i;
	for (i=0;i<len;i++)
	{
		while (uartSendBuffer(UART1, &buffer[i], 1)==FALSE);
	}
}

int main(void)
{
	uartInit();  // initialize the UART (serial port)
    uartSetBaudRate(0, 38400); // set UARTE speed, for Bluetooth
    uartSetBaudRate(1, 115200); // set UARTD speed, for USB connection
    uartSetBaudRate(2, 38400); // set UARTH speed
    uartSetBaudRate(3, 38400); // set UARTJ speed, for Blackfin

	rprintfInit(uart1SendByte);// initialize rprintf system and configure uart1 (USB) for rprintf

	configure_ports();

	timer0Init();

	uartSetRxHandler(1, &receive_data);

	//initialize ISR:
	UCSR1B |= (1 << RXCIE1); // Enable the USART Recieve Complete interrupt (USART_RXCn)
	sei();//turn on the Global Interrupt Enable flag

  	unsigned char buffer[8];
    unsigned char command;
    unsigned char mailbox=0;
	unsigned long lvalue;
	unsigned char *tvalue;
	unsigned char crc;
    int hold;
	int i,j;

  	for (i=0;i<20;i++) { mailboxToSend[i]=0; mailboxSendFlag[i]=0; }

	while(1)
	{
	    // command is the only byte that should have bit 7 set.
	    do
	    {
	      command = getByte();
	    }
	    while ((command&128)==0);

	    crc = command;

	    command^=128;

	    switch (command)
	    {
	      // init
	      case  MCU_GET_ID:
		  	write((char *)"MCUC", 4);
	      break;
	      case  MCU_SET_NUMBER_MAILBOX:
	        if ((buffer[0] = getByte())&128) continue;
	        crc ^= buffer[0];
	        if ((buffer[1] = getByte())&128) continue;
	        crc ^= buffer[1];
	        if ((buffer[2] = getByte())&128) continue;
	        crc ^= buffer[2];
	        if ((buffer[3] = getByte())&128) continue;
	        crc ^= buffer[3];
	        if ((buffer[4] = getByte())&128) continue;
	        crc ^= buffer[4];
	        if ((buffer[5] = getByte())&128) continue;
	        crc ^= buffer[5];
	        if ((buffer[6] = getByte())&128) continue;
	        if ((crc&127)!=buffer[6]) continue;
	        intValue[buffer[0]] = (unsigned long)buffer[1]|((unsigned long)buffer[2]<<7)|((unsigned long)buffer[3]<<14)|((unsigned long)buffer[4]<<21)|((unsigned long)buffer[5]<<28);
	        mailboxToSend[buffer[0]]=0;
	      break;
	      case  MCU_SET_LOGIC_MAILBOX:
	        if ((buffer[0] = getByte())&128) continue;
	        crc ^= buffer[0];
	        if ((buffer[1] = getByte())&128) continue;
	        crc ^= buffer[1];
	        if ((buffer[2] = getByte())&128) continue;
	        if ((crc&127)!=buffer[2]) continue;
	        logicValue[buffer[0]] = buffer[1];
	        mailboxToSend[buffer[0]]=0;
	      break;
	      case  MCU_SET_TEXT_MAILBOX:
	        if ((mailbox = getByte())&128) continue;
	        crc ^= mailbox;
	        hold = textStoreTop;
	        textStoreTop--;
	        do
	        {
	          textStoreTop++;
	          textStore[textStoreTop] = getByte();
	          crc^=textStore[textStoreTop];
	        }
	        while ((textStoreTop<1024)&&(textStore[textStoreTop]));
	        textStoreTop++;
	        if ((crc&127)!=getByte()) { textStoreTop=hold; continue; }
	        textValue[mailbox] = &textStore[hold];
	        mailboxToSend[mailbox]=0;
	      break;
	      //stream values to mailbox
	      case  MCU_GET_NUMBER_MAILBOX:
	        mailbox = getByte();
	        if (((crc^mailbox)&127)!=getByte()) continue;
	        mailboxToSend[mailbox]=MCU_GET_NUMBER_MAILBOX;
	      break;
	      case  MCU_GET_LOGIC_MAILBOX:
	        mailbox = getByte();
	        if (((crc^mailbox)&127)!=getByte()) continue;
	        mailboxToSend[mailbox]=MCU_GET_LOGIC_MAILBOX;
	      break;
	      case  MCU_GET_TEXT_MAILBOX:
	        mailbox = getByte();
	        if (((crc^mailbox)&127)!=getByte()) continue;
	        mailboxToSend[mailbox]=MCU_GET_TEXT_MAILBOX;
	      break;
	      case  MCU_END_FRAME:
	        processMailboxes();
	        // note that Text values are lost after being processed as we
	        // will reuse the buffer for the next frame
	        textStoreTop=0;
	      break;
		}

		for (i=0;i<20;i++)
		{
		  	if (mailboxToSend[i]&&mailboxSendFlag[i])
		  	{
		    	switch (mailboxToSend[i])
		    	{
		        case  MCU_GET_NUMBER_MAILBOX:
		          lvalue = intValue[i];

		          send_buffer[0]=MCU_GET_NUMBER_MAILBOX|128;
		          send_buffer[1]=i;
		          send_buffer[2]=lvalue&127;
		          send_buffer[3]=(lvalue>>7)&127;
		          send_buffer[4]=(lvalue>>14)&127;
		          send_buffer[5]=(lvalue>>21)&127;
		          send_buffer[6]=(lvalue>>28)&127;
		          send_buffer[7]=(send_buffer[0]^send_buffer[1]^send_buffer[2]^send_buffer[3]^send_buffer[4]^send_buffer[5]^send_buffer[6])&127;

			  	  write((char *)send_buffer, 8);
		        break;
		        case  MCU_GET_LOGIC_MAILBOX:
		          send_buffer[0]=MCU_GET_LOGIC_MAILBOX|128;
		          send_buffer[1]=i;
		          send_buffer[2]=logicValue[i]&127;
		          send_buffer[3]=(send_buffer[0]^send_buffer[1]^send_buffer[2])&127;

			  	  write((char *)send_buffer, 4);
		        break;
		        case  MCU_GET_TEXT_MAILBOX:
		          tvalue = textValue[i];

		          send_buffer[0]=MCU_GET_TEXT_MAILBOX|128;
		          send_buffer[1]=i;
		          crc = send_buffer[0]^send_buffer[1];
		          j=0;
		          while ((tvalue[j]!=0)&&(j<254))
		          {
		            tvalue[j] &= 127;
		            crc ^= tvalue[j];
		            j++;
		          }
		          j++;

		          send_buffer[2] = crc&127;

			  	  write((char *)send_buffer, 2);
			  	  write((char *)tvalue, j);
			  	  write((char *)&send_buffer[2], 1);
		        break;
		      	}

				mailboxSendFlag[i]=0;
			}
		}
	}
}

// called after mailboxes from the PC have been sent ... this is where your code will go!
void processMailboxes()
{
  // variables value will be in one of intValue, textValue or logicValue
  // arrays and address by the mailbox number. Note the variable name is
  // NOT sent by identified by the corresponding mailbox number
  // as seen in the MCU Communicator GUI

  // we have set mailbox 0 to a logic value coming from RR. In this case
  // we attach it to the IMAGE_COUNT variable so we flash for every few
  // frames.

  if (intValue[0]&4)
	LED_on();
  else
	LED_off();


  // To send a value back to mailbox #1 specify it as a Get in the MCU Communicator
  // interface and type in a variable name that will hold that value. Indicate its type
  // as Number and then use the following to send a value of IMAGE_COUNT+100 back to the PC
  // into that variable. Don't forget to set the flag!
  intValue[1]=intValue[0]+100;
  mailboxSendFlag[1] = 1;

  // To send back a string configure the MCU Communicator to a text type and use
  textValue[2] = (unsigned char *)"this is a test";
  mailboxSendFlag[2] = 1;

  // Now we convert mailbox 3's string into an uppercase version
  // of it and send it back via mailbox 4 into another string
  // variable. Note this is not a very reliable uppercase conversion
  // but just illustrates the point of string manipulation
  static unsigned char replyStr[256];
  int i;
  for (i=0;textValue[3][i];i++)
  {
    if (textValue[3][i]!=' ')
      replyStr[i]=textValue[3][i]+('A'-'a');
    else
      replyStr[i]=' ';
  }
  replyStr[i]=0;

  textValue[4]=replyStr;
  mailboxSendFlag[4] = 1;

  // and set mailbox 5 to the length of this string
  intValue[5] = i;
  mailboxSendFlag[5] = 1;

  // invert the logic on mailbox 6
  logicValue[7] = logicValue[6]^1;
  mailboxSendFlag[7] = 1;
}



